/** 
 * (C) Copyright IBM Corp. 2011
 */
package $packageName$;

import com.ibm.pdp.engine.IBuilderTag;
import com.ibm.pdp.engine.IGenInfoBuilder;
import com.ibm.pdp.engine.IMicroPattern;
import com.ibm.pdp.maf.rpp.kernel.RadicalElement;
import com.ibm.pdp.maf.rpp.pac.dataelement.DataElement;
import com.ibm.pdp.maf.rpp.service.IMAFModelService;
import com.ibm.pdp.maf.rpp.service.MAFModelService;
import com.ibm.pdp.rpp.micropattern.AbstractGlobalMicroPatternHandler;


/**
 * This is a global  Micro Pattern that displays the label of a Data Element in a new function
 * just after the PROCEDURE-DIVISION.
 * 
 * You can add those lines anywhere into your COBOL code.
 * 
      *{MPG Project="MyProject" Package="MyPaclage"  Code="MyDECode"
      *} 
 */
public class  $className$ extends AbstractGlobalMicroPatternHandler {
	private static final String TYPE_FUNCTION = "function"; //$NON-NLS-1$
	
	/**
	 * Returns the identifier of the Micro Pattern.
	 * 
	 */
	@Override
	public String getId() {
		return "MPG";
	}

	/**
	 * Called once per Micro Pattern having same ID and name space found in the source code
	 * 
	 * @param microPattern The micro pattern
	 * @param genInfoBuilder The GenInfo Builder
	 * @param radicalElement The radical element source of the generation
	 * @return The text to insert into the micro pattern declaration
	 */	
	public String process(final IMicroPattern microPattern,  final  IGenInfoBuilder genInfoBuilder,  final RadicalElement radicalElement )  {
		IMAFModelService mafService = MAFModelService.getInstance();
		String project = microPattern.getAttribute("Project");
		String pack = microPattern.getAttribute("Package");
		String code = microPattern.getAttribute("Code");
		if ( project == null || pack == null || code == null ) {
			microPattern.getProcessingContext().setStatus(
					IMicroPattern.WARNING_RAISED,
					microPattern.getLocation().getBeginIndex(),
					"Project/Package/Code Parameters not found!!!",
					"Please, review this Micro-Pattern declaration");
			return null;
		}
		
		// Retrieve the Data Element to display
		DataElement dataElement = mafService.getDataElement( project, pack, code);		
		if ( dataElement == null ) {
			microPattern.getProcessingContext().setStatus(
					IMicroPattern.WARNING_RAISED,
					microPattern.getLocation().getBeginIndex(),
					"Data Element not found!!!",
					"Please, review this Micro-Pattern declaration");
			return null;			
		}
		
		IBuilderTag procDivisionTag = genInfoBuilder.tagFromName("PROCEDURE-DIVISION");
		if ( procDivisionTag != null ) {
			IBuilderTag functionTag = genInfoBuilder.tagFromName("F00");
			if ( functionTag == null )
				functionTag = genInfoBuilder.addTag(procDivisionTag.getEndIndex(), procDivisionTag.getEndIndex(), "F00");
			String newCode = 
				"       F00.      EXIT.\n"+
				"           DISPLAY \"" + dataElement.getLabel() + "\".\n"+
				"       F00-FN.   EXIT.\n";
				
			functionTag.setText(newCode);
			// If the micro pattern is declared in the COBOL code then add micro pattern delimiters.
			// Used by the runtime.
			if (microPattern.getProcessingContext().inUserCode()) {
				functionTag.setTextBefore(getBeginFragmentDeclaration(microPattern, TYPE_FUNCTION));
				functionTag.setTextAfter(getEndFragmentDeclaration(microPattern));
			}			
		} else {
			microPattern.getProcessingContext().setStatus(
					IMicroPattern.WARNING_RAISED,
					microPattern.getLocation().getBeginIndex(),
					"No procedure division found!!!",
					"Please, review this Micro-Pattern declaration");			
			return null;
		}

		registerReference(microPattern, dataElement);

		StringBuffer result = new StringBuffer();
		// If the micro pattern is declared in the COBOL code then add the declaration header.
		boolean inUserCode = microPattern.getProcessingContext().inUserCode();
		if(inUserCode)
			result.append(microPattern.getOriginalHeaderDeclaration()+"\n");
		result.append("      * Generation OK\n");
		// If the micro pattern is declared in the COBOL code then add the declaration footer.
		if(inUserCode)
			result.append(microPattern.getClosingSequence());
		// Register the data element reference for this cobol file
		return result.toString();
	}
}