/** 
 * (C) Copyright IBM Corp. 2011
 */
package $packageName$;

import com.ibm.pdp.engine.IMicroPattern;
import com.ibm.pdp.maf.rpp.kernel.RadicalElement;
import com.ibm.pdp.maf.rpp.pac.dataelement.DataElement;
import com.ibm.pdp.maf.rpp.service.IMAFModelService;
import com.ibm.pdp.maf.rpp.service.MAFModelService;
import com.ibm.pdp.rpp.micropattern.AbstractSimpleMicroPatternHandler;

/**
 * This is a simple Micro Pattern that displays the label of a Data Element.
 * 
 * You can add those lines into your COBOL code.
 * 
      *{MP Project="MyProject" Package="MyPaclage"  Code="MyDECode"
      *} 
 */
public class $className$ extends AbstractSimpleMicroPatternHandler {
	
	/**
	 * Returns the identifier of the Micro Pattern.
	 * 
	 */
	@Override
	public String getId() {
		return "MP";
	}

	/**
	 * Called once per Micro Pattern having same ID and name space found in the source code
	 * 
	 * @param microPattern The Micro Pattern
	 * @param radicalElement The radical element source of the generation
	 * @return a String to replace the current Micro Pattern code in the source
	 */		
	@Override
	public String process(final IMicroPattern microPattern, final RadicalElement radicalElement) {
		IMAFModelService mafService = MAFModelService.getInstance();
		String project = microPattern.getAttribute("Project");
		String pack = microPattern.getAttribute("Package");
		String code = microPattern.getAttribute("Code");
		if ( project == null || pack == null || code == null ) {
			microPattern.getProcessingContext().setStatus(
					IMicroPattern.WARNING_RAISED,
					microPattern.getLocation().getBeginIndex(),
					"Project/Package/Code Parameters not found!!!",
					"Please, review this Micro-Pattern declaration");
			return null;
		}
		
		// Retrieve the Data Element to display
		DataElement dataElement = mafService.getDataElement( project, pack, code);		
		if ( dataElement == null ) {
			microPattern.getProcessingContext().setStatus(
					IMicroPattern.WARNING_RAISED,
					microPattern.getLocation().getBeginIndex(),
					"Data Element not found!!!",
					"Please, review this Micro-Pattern declaration");
			return null;			
		}
		
		StringBuffer result = new StringBuffer();
		
		// If the micro pattern is declared in the COBOL code then add the declaration header.
		boolean inUserCode = microPattern.getProcessingContext().inUserCode();
		if(inUserCode)
			result.append(microPattern.getOriginalHeaderDeclaration());
		
		result.append("       DISPLAY \"" + dataElement.getLabel() + "\".\n");
		// If the micro pattern is declared in the COBOL code then add the declaration footer.
		if(inUserCode)
			result.append(microPattern.getClosingSequence());
		// Register the data element reference for this cobol file
		registerReference(microPattern, dataElement);
		return result.toString();

	}
}