/** 
 * (C) Copyright IBM Corp. 2011
 */
package $packageName$;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.ibm.pdp.maf.rpp.kernel.UserEntity;
import com.ibm.pdp.maf.rpp.pac.dataelement.DataElement;
import com.ibm.pdp.maf.rpp.service.IMAFModelService;
import com.ibm.pdp.maf.rpp.service.MAFModelService;
import com.ibm.pdp.product.tools.extension.IRppOption;
import com.ibm.pdp.product.tools.extension.IRppSubCommand;
import com.ibm.pdp.product.tools.extension.IRppSubCommandResult;
import com.ibm.pdp.product.tools.extension.RppOption;
import com.ibm.pdp.product.tools.extension.RppSubCommandResult;

/**
 * The is a simple rpp.bat command line extension that add two new sub commands.
 * 
 *  - The first sub command print the name of a data element in the logger
 *  - The second sub command print the name of a user entity in the logger.
 */
public class $className$ implements IRppSubCommand {
	
	/**
	 * List of new sub commands.
	 */
	static List<IRppOption> subCommandOptions = new ArrayList<IRppOption>();
	static {
		subCommandOptions.add ( 
				RppOption.createCommandOption("command1", "c1", "First sub command") );
		subCommandOptions.add ( 
				RppOption.createCommandOption("command2", "c2", "Second sub command") );
	}
	
	/**
	 * Options for the first sub command.
	 */
	static List<IRppOption> command1Arguments = new ArrayList<IRppOption>();
	static {
		command1Arguments.add(RppOption.createOption("project",null,"Project Name",false,false,true,null));
		command1Arguments.add(RppOption.createOption("package",null,"Package Name",false,false,true,null));
		command1Arguments.add(RppOption.createOption("name",null,"Data Element Name",false,false,true,null));
	}
	
	/**
	 * Options for the second sub command.
	 */
	static List<IRppOption> command2Arguments = new ArrayList<IRppOption>();
	static {
		command2Arguments.add(RppOption.createOption("project",null,"Project Name",false,false,true,null));
		command2Arguments.add(RppOption.createOption("package",null,"Package Name",false,false,true,null));
		command2Arguments.add(RppOption.createOption("meta",null,"Meta Entity Name",false,false,true,new String[] {"DEMO"}));
		command2Arguments.add(RppOption.createOption("name",null,"User Entity Name",false,false,true,null));
	}	
	
	@Override
	public List<IRppOption> getSubCommandOptions() {
		return subCommandOptions;
	}

	@Override
	public List<IRppOption> getOptionsForSubCommand(IRppOption subCommandOption) {
		if ( subCommandOption == subCommandOptions.get(0))
			return command1Arguments;
		else
			return command2Arguments;
	}

	@Override
	public IRppSubCommandResult execute(IRppOption subCommandOption,
			Map<String, List<String>> subCommandOptionsValues, Logger logger ) {
		if ( subCommandOption == subCommandOptions.get(0)) {
			IMAFModelService service = MAFModelService.getInstance();
			String projectName = subCommandOptionsValues.get("project").get(0);
			String packageName = subCommandOptionsValues.get("package").get(0);
			String dataElementName = subCommandOptionsValues.get("name").get(0);
			DataElement dataElement =
					service.getDataElement(projectName, packageName, dataElementName);
			if ( dataElement != null ) {
				logger.log(Level.INFO,"Found DataElement : " + dataElement.getName() );
				return new RppSubCommandResult(0, null);
			}
			else {
				logger.log(Level.SEVERE,"DataElement Not Found  : " + dataElementName );
				return new RppSubCommandResult(1, "DataElement Not Found");
			}
		} else {
			IMAFModelService service = MAFModelService.getInstance();
			String projectName = subCommandOptionsValues.get("project").get(0);
			String packageName = subCommandOptionsValues.get("package").get(0);
			String metaEntityName = subCommandOptionsValues.get("meta").get(0);
			String userEntityName = subCommandOptionsValues.get("name").get(0);
			UserEntity userEntity =
					service.getUserEntity(projectName, packageName, metaEntityName, userEntityName);
			if ( userEntity != null ) {
				logger.log(Level.INFO,"Found UserEntity : " + userEntity.getName() );
				return new RppSubCommandResult(0, null);
			}
			else {
				logger.log(Level.SEVERE,"UserEntity Not Found  : " + userEntityName );
				return new RppSubCommandResult(2, "UserEntity Not Found");
			}
		}
	}
}
